/**
 * @file spinc.cpp 
 * Convert Spin DAT PASM to a C ASCII-HEX array.
 * Copyright (c) 2008, Steve Denson
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include "spinc.h"

/**
 * load spin binary from filename
 * @param name - path and filename
 * @returns pointer to binary or 0 on failure.
 */
static char* loadSpinBinary(char* name)
{
    FILE* fp;
    char* rp = 0;
    int   size = 0;
    int   ii;
    char* bin = 0;
    char  hdr[sizeof(SpinHdr_st)];
    SpinHdr_st* sp = 0;

    fp = fopen(name, "rb");
    if(!fp) {
        printf("Can't find spin binary '%s'.\n", name);
        goto exit;
    }
    else {
        for(ii = 0; ii < 16; ii++) {
            hdr[ii] = fgetc(fp);
            if (feof(fp)) {
                printf("Bad spin binary header. Unexpected end of file in '%s'\n", name);
                goto exit;
            }
        }
        sp = (SpinHdr_st*) hdr;
        if(sp->objstart != SPIN_OBJSTART) {
            printf("Bad spin object start in '%s'. Expecting a spin binary for conversion.\n", name);
            goto exit;
        }
        size = sp->varstart;
        fclose(fp);
        fp = fopen(name, "rb");
        rp = (char*) malloc(size);
//        for(ii = 0; ii < size; ii++) {
//            rp[ii] = fgetc(fp);
//        }
        ii = fread(rp,1,size,fp);
        if (ii <= 0) {
            printf("Can't read spin binary '%s' [%d].\n", name, ii);
            goto exit;
        }
    }
exit:
    if(fp) fclose(fp);
    return rp;
}

/**
 * print C array
 * @param p - pointer to spin binary image
 * @param name - array name
 */
static void printCarray(char* p, char* name)
{
    int ii;
    int datstart = 0;
    int datlen = 0;
    SpinHdr_st* sp = (SpinHdr_st*) p;
    SpinObj_st* op = (SpinObj_st*) &sp->code;

    datstart = (int) (((int)op-(int)sp)+(op->objcnt+op->pubcnt)*4);
    datlen   = (int) (sp->pubstart-datstart);
    printf("long %s_array[] =\n{\n    ", name);
    for(ii = 0; ii < datlen; ii+=4) {
        if (ii && ((ii % 16) == 0)) {
            printf("\n    ");
            //printf("0x%04x: ",ii+datstart);  // save for debug
        }
        printf("0x%08x, ", *(int*)(p+ii+datstart));
    }
    printf(" 0\n};\n");
}

/**
 * print program usage
 * @param name - filename
 */
static void printUsage(char* name)
{
    printf("Convert Spin DAT PASM to a C ASCII-HEX array.\n");
    printf("%s <filename>\n", name);
}

/**
 * chop path from filename
 * @param name - path and filename
 * @returns filename
 */
static char* chopPath(char* name)
{
    int ii = 0;
    int len = strlen(name);
    char* start = name;
    char* p = 0;
    int sep = 0;

    if(strchr(name, '\\'))  sep = '\\';
    if(strchr(name, '/'))   sep = '/';

    while((start+len-name) > 0) {
        p = strchr(name+1, sep);
        if (!p) break;
        name = p+1;
    }
    return name;
}

/**
 * chop extension from filename
 * @param name - path and filename
 * @returns filename
 */
static char* chopExtention(char* name)
{
    char* p  = strchr(name, '.');
    int   ii = (int)p-(int)name;
    if (p && ii) name[ii] = '\0';
    return name;
}

int main(int argc, char* argv[])
{
    char* spinptr = 0;
    char* spinbin = 0;
    time_t mytime = 0;
    struct tm* today;
    time(&mytime);
    today = localtime(&mytime);

    if(argc < 2) {
        printUsage(argv[0]);
        return 0;
    }
    else {
        spinptr = loadSpinBinary(argv[1]);
        if(spinptr) {
            spinbin = chopExtention((argv[1]));
            printf("/** \n");
            printf(" * @file %s_array.h\n", spinbin); 
            printf(" * Created with spin.binary PASM to C Array Converter.\n");
            printf(" * Copyright (c) %d, John Doe\n", today->tm_year+1900);
            printf(" */\n");
            printCarray(spinptr, spinbin);
        }
    }
    return 0;
}

